--- old/usr/src/cmd/rmvolmgr/rmm_common.c	2014-05-04 18:28:36.795228060 +0100
+++ new/usr/src/cmd/rmvolmgr/rmm_common.c	2014-05-04 18:28:36.803683284 +0100
@@ -17,10 +17,12 @@
  * information: Portions Copyright [yyyy] [name of copyright owner]
  *
  * CDDL HEADER END
- */
-/*
+ *
+ *
  * Copyright 2009 Sun Microsystems, Inc.  All rights reserved.
  * Use is subject to license terms.
+ *
+ * Copyright 2014 Andrew Stormont.
  */
 
 #include <stdio.h>
@@ -807,6 +809,8 @@
 	DBusError	error;
 	boolean_t	ret = B_FALSE;
 
+	dprintf("rmm_action_one %s %s\n", name, action_strings[action]);
+
 	if (strcmp(name, dev) == 0) {
 		(void) snprintf(dev_str, sizeof (dev_str), name);
 	} else {
@@ -821,16 +825,10 @@
 		break;
 	case INSERT:
 	case REMOUNT:
-		if (libhal_volume_is_mounted(v)) {
-			goto done;
-		}
 		ret = rmm_hal_mount(hal_ctx, udi,
 		    opts, num_opts, mountpoint, &error);
 		break;
 	case UNMOUNT:
-		if (!libhal_volume_is_mounted(v)) {
-			goto done;
-		}
 		ret = rmm_hal_unmount(hal_ctx, udi, &error);
 		break;
 	case CLOSETRAY:
@@ -919,6 +917,18 @@
 		action = UNMOUNT;
 	}
 
+	/*
+	 * We can't mount or unmount a drive that has no volumes.
+	 * Either the media isn't inserted or it's not formatted
+	 */
+	if (volumes == NULL && (action != EJECT && action != CLOSETRAY)) {
+		(void) fprintf(stderr, libhal_drive_requires_eject(d) ? 
+		    gettext("no volumes in '%s' to %s\n") :
+		    gettext("no volumes on '%s' to %s\n"),
+		    name, action_strings[action]);
+		goto out;
+	}
+
 	/* per drive action */
 	if ((action == EJECT) || (action == CLOSETRAY)) {
 		ret = rmm_action_one(hal_ctx, name, action, d_dev, d_udi, NULL,
@@ -962,8 +972,10 @@
 	}
 
 out:
-	rmm_volumes_free(volumes);
-	libhal_drive_free(d);
+	if (volumes != NULL)
+		rmm_volumes_free(volumes);
+	if (d != NULL)
+		libhal_drive_free(d);
 
 	return (ret);
 }
